package characters;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import swinggui.JTurtleWorld;
import swinggui.SpriteInterface;
import swinggui.WorldInfo;

public class Sprite implements SpriteInterface {

	
	Point location = new Point (0,0);	
	int rotation = 0;
	int stepSize = 12;
    BufferedImage img = null;	
    WorldInfo worldInfo;
    
	
	public Sprite () {
		
		   File f = new File("img\\" + fileName());

		   try {
		       img = ImageIO.read(f);
		   } catch (IOException e) {
			  
		   }
		
		
	}
	
	public void setWorldInfo (WorldInfo info) {
		worldInfo = info;
	}
	
	
	public String fileName()     { return "joker.gif";}
	
	public BufferedImage getImage () {
		return img;
	}
	
	public void setLocation (int x, int y) {
		location = new Point (x,y);
	}
	
	public Point getLocation () {return location; }
	
	public Point getCenter() {
		
		
		int x = location.x + img.getWidth() / 2;
		int y = location.y + img.getHeight() / 2;
		
		return new Point ( x, y);
		
		
	}
	
	public void setRotation (int degree) { rotation  = degree; }
	
	public double getRadianRotation () {
		 return Math.toRadians(rotation);
	}
	

    
	// checks for collision and moves one step in the current direction
	// using the current step size
	// sub classes should call super.step() to check for collision and perform one step
	// changes of directions, change of speed, or jumps should be 
	// implemented in sub classes
    
	public void step () {
		 // Kollision mit Auenkanten?
        checkCollision();
		// move forward:
		 double r = Math.toRadians(Math.abs(rotation)-90);
	     location.x += Math.round(stepSize * Math.cos(r));  
	     location.y += Math.round(stepSize * Math.sin(r));		
	     
		
		
	}
	
	
	// collision reaction are empty implementations, it is up to the
	// sub classes to implement any reaction if required
	public void onTopCollision() {

	}
	
	public void onBottomCollision() {

	}
	
	public void onLeftCollision () {
		
			
	}
	
	public void onRightCollision () {

	}
	
	
	// collisioin detection with world bounds is the same for all sprites
	// reaction is delegated to methods which should be overriden in sub classes if needed
	private void checkCollision() {
		 if (worldInfo == null) return;
		 if (location.x < worldInfo.getLeft() ) onLeftCollision();
		 if (location.x > worldInfo.getRight()) onRightCollision() ;
		 if (location.y < worldInfo.getTop() ) onTopCollision();
		 if (location.y > worldInfo.getBottom()) onBottomCollision();

	}
	
	
	// same helper method as in Stupid Sprite - rotates to a defined point
	protected void rotateToTarget (Point target) {
		
		Point center = this.getCenter();	
		
    	if (target.x > center.x && target.y < center.y) {
     	   double r = Math.atan2( center.x - target.x , target.y - center.y );	
     	   rotation = (int) Math.round( Math.toDegrees(r)) + 180;
     	     	  	
     	}
     	
    	
         if (target.x > center.x && target.y > center.y) {

            double r = Math.atan2( target.y - center.y ,  center.x - target.x   );	
      	   rotation = 270 - (int) Math.round( Math.toDegrees(r)) 	;
         }
         
         if (target.x < center.x && target.y > center.y) {
             double r = Math.atan2(  center.x - target.x , target.y - center.y  );	
       	   rotation = 180 + (int) Math.round( Math.toDegrees(r)) 	;	
         }
         
         if (target.x < center.x && target.y < center.y) {
             double r = Math.atan2( center.y - target.y , center.x - target.x  );	
       	   rotation = 270 + (int) Math.round( Math.toDegrees(r)) 	;	      	
         	
         }		
		
		
		
	}
	
	
	
}
