package decorators;

import java.awt.Color;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Graphics2D;

import swinggui.Pen;
import swinggui.SpriteInterface;

public class PenDecoration extends SpriteDecorator implements Pen{

	
	private Polygon strokes = new Polygon();
	private Color color;
	
	public PenDecoration (SpriteInterface decoratedSprite , Color c) {
		  this.decoratedSprite = decoratedSprite;
		  color = c;
	}
	
	// Overrides the step() method to extend the functionality of the decorated sprite
	@Override
	public void step () {
	  
		decoratedSprite.step(); // this is important to ensure that the original functionality is preserved
		
		if (strokes != null) {
			Point center = decoratedSprite.getCenter();
			strokes.addPoint(  center.x  , center.y  );		
		}
	
	}
	
	
	// implementation of Pen interface
	// draws the strokes of this pen to the provided g2d graphic context
	public void paintStrokes (Graphics2D g2d) {
		 Color oldColor =  g2d.getColor();
		 g2d.setColor(color);
		 g2d.drawPolyline(strokes.xpoints, strokes.ypoints, strokes.npoints);
		 g2d.setColor(oldColor);
	}
	
	
}
