package stupid;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import swinggui.SpriteInterface;
import swinggui.WorldInfo;
import characters.Sprite;

public class StupidSprite implements SpriteInterface {

	// location and configuration of the sprite
	Point location = new Point (0,0);	
	int rotation = 0;
	int stepSize = 12;
	int stepCount = 0;
    BufferedImage img = null;	
    WorldInfo worldInfo;
    
    // in case we want to hunt another sprite
    private SpriteInterface targetSprite;
    
	// different characters:
	public static final int TURTLE  = 0;
	public static final int DOG = 1;
	public static final int CAT = 2;
	public static final int GEKKO = 3;
	
	private int spriteType = TURTLE;
	
	public StupidSprite (int type, SpriteInterface targetSprite) {
		this (type);
		this.targetSprite = targetSprite;
		
	}
	
	public StupidSprite (int type) {
		
		 spriteType = type;
		 
		 
		 String fileName = "joker.gif"; // fallback
		 
		 if (type == TURTLE) fileName = "turtleBlack.gif";
		 if (type == DOG) fileName = "dogRed.gif";
		 if (type == CAT) fileName = "catGreen.gif";
		 if (type == GEKKO)  fileName ="gekkoBlue.gif";
		 
		    
		   File f = new File("img\\" + fileName);

		   try {
		       img = ImageIO.read(f);
		   } catch (IOException e) {
			   

		   }
		
	}
	
	
	public void step () {
		 // Kollision mit Auenkanten?
         checkCollision();
         
         
		 if (spriteType == TURTLE) {
			    Point target = worldInfo.getTarget();
			    rotateToTarget(target); 
		 }
		 
		 if (spriteType == DOG) {
			 rotateToTarget(targetSprite.getCenter());
		 }
		 
		 
		 if (spriteType == CAT) {
			 int y = worldInfo.getTarget().y;
			 if ( y != location.y) {
				 location.y =  y;
			 }
			 
			 
		 }
		 
		 if (spriteType == GEKKO) {
			   stepCount++;
			   if (stepCount == 10) {
				   stepCount = 0;
				   rotation += 20;
			   }
		 }
        
		// move forward:
		 double r = Math.toRadians(Math.abs(rotation)-90);
	     location.x += Math.round(stepSize * Math.cos(r));  
	     location.y += Math.round(stepSize * Math.sin(r));		     

		
	}
	
	
	
	public void onTopCollision() {
		
		 if (spriteType == TURTLE) {
			 location.y = worldInfo.getCenterY();
		 }
		 if (spriteType == DOG) {
			    location.y = worldInfo.getCenterY();
		 }
		 if (spriteType == CAT) {
			    location.y = worldInfo.getBottom();
		 }
		 
		 if (spriteType == GEKKO) {
				rotation += 180;
		 }		
		
	}
	
	public void onBottomCollision() {
		 if (spriteType == TURTLE) {
			 location.y = worldInfo.getCenterY();
		 }
		 if (spriteType == DOG) {
			    location.y = worldInfo.getCenterY();
		 }
		 if (spriteType == CAT) {
			    location.y = 0;
		 }
		 
		 if (spriteType == GEKKO) {
				rotation += 180;
		 }
	}
	
	public void onLeftCollision() {
		 if (spriteType == TURTLE) {
			    location.x = worldInfo.getCenterX();
		 }
		 if (spriteType == DOG) {
			    location.x = worldInfo.getCenterX();
		 }
		 if (spriteType == CAT) {
			 location.x = worldInfo.getRight();
		 }
		 
		 if (spriteType == GEKKO) {
				rotation += 180; 
		 }
	}
	
	public void onRightCollision() {
		 if (spriteType == TURTLE) {
			    location.x = worldInfo.getCenterX(); 
		 }
		 if (spriteType == DOG) {
			 location.x = worldInfo.getCenterX();
		 }
		 if (spriteType == CAT) {
			 location.x = 0;
		 }
		 
		 if (spriteType == GEKKO) {
				rotation += 180;	 
		 }
	}	
	
	
	
	private void checkCollision() {
		 if (worldInfo == null) return;
		 if (location.x < worldInfo.getLeft() ) onLeftCollision();
		 if (location.x > worldInfo.getRight()) onRightCollision() ;
		 if (location.y < worldInfo.getTop() ) onTopCollision();
		 if (location.y > worldInfo.getBottom()) onBottomCollision();

	}




	@Override
	public double getRadianRotation () {
		 return Math.toRadians(rotation);
	}


	@Override
	public Point getLocation () {return location; }


	@Override
	public Point getCenter() {
		
		
		int x = location.x + img.getWidth() / 2;
		int y = location.y + img.getHeight() / 2;
		
		return new Point ( x, y);
		
		
	}


	@Override
	public BufferedImage getImage () {
		return img;
	}


	@Override
	public void setLocation (int x, int y) {
		location = new Point (x,y);
	}


	@Override
	public void setRotation (int degree) { rotation  = degree; }


	@Override
	public void setWorldInfo (WorldInfo info) {
		worldInfo = info;
	}
	
	

	// helper method that rotates on object towards a target point
	void rotateToTarget (Point target) {
		
		Point center = this.getCenter();	
		
    	if (target.x > center.x && target.y < center.y) {
     	   double r = Math.atan2( center.x - target.x , target.y - center.y );	
     	   rotation = (int) Math.round( Math.toDegrees(r)) + 180;
     	     	  	
     	}
     	
    	
         if (target.x > center.x && target.y > center.y) {

            double r = Math.atan2( target.y - center.y ,  center.x - target.x   );	
      	   rotation = 270 - (int) Math.round( Math.toDegrees(r)) 	;
         }
         
         if (target.x < center.x && target.y > center.y) {
             double r = Math.atan2(  center.x - target.x , target.y - center.y  );	
       	   rotation = 180 + (int) Math.round( Math.toDegrees(r)) 	;	
         }
         
         if (target.x < center.x && target.y < center.y) {
             double r = Math.atan2( center.y - target.y , center.x - target.x  );	
       	    rotation = 270 + (int) Math.round( Math.toDegrees(r)) 	;	      	
         	
         }		
		
		
		
	}
	
	
	
	
}
