package swinggui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;










import java.util.ArrayList;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.Timer;

import strategycharacters.CatSprite;
import strategycharacters.DogSprite;
import strategycharacters.GekkoSprite;
import strategycharacters.SmartSprite;
import strategycharacters.StrategySpritesFactory;
import strategycharacters.TurtleSprite;
import stupid.StupidSprite;
import stupid.StupidSpriteFactory;
import characters.AnimalFactory;
import characters.Turtle;
import characters.Dog;
import characters.Cat;
import characters.Gekko;
import characters.Sprite;
import decorators.LogDecoration;
import decorators.PenDecoration;

@SuppressWarnings("serial")
public class JTurtleWorld extends JComponent {
	
    // stores all sprites that can be painted on the canvas
	ArrayList <SpriteInterface> sprites = new ArrayList<SpriteInterface> ();
	// stores pens that provide polygons of strokes to be pained
	ArrayList <Pen> pens = new ArrayList<Pen> ();
	// provides information about world size to sprite implementations
	WorldInfo world = new WorldInfo (600,600);
	 

	
	// paints the component
    public void paintComponent(Graphics g) {

    	 Graphics2D g2d = (Graphics2D) g; // cast to access 2d graphic methods
    	 
    	 // draw world bounds
    	 Rectangle bounds = world.getBounds();
    	 g2d.drawRect(bounds.x , bounds.y , bounds.width, bounds.height);
    	 
    	 // draw strokes
    	 Iterator<Pen>allPens = pens.iterator();
    	 while (allPens.hasNext()) {
    		 	allPens.next().paintStrokes(g2d);
    	 }
    	 
    	 // draw sprites
    	 Iterator <SpriteInterface> allSprites = sprites.iterator();
    	 while (allSprites.hasNext()) {
    		 drawSprite(g2d, allSprites.next());
    	 }

         // draw target
    	 g2d.setColor(Color.blue);
    	 g2d.fillRect( world.getTarget().x-2, world.getTarget().y-2, 4, 4);
  

 	 
    }
    
    // draw a single sprite
    private void drawSprite (Graphics2D g2d ,  SpriteInterface sprite) {
    	
    	// get coordinates of sprite
    	Point center = sprite.getCenter();
    	Point location = sprite.getLocation();
    	
    	// Use an affine transform to draw the sprite image rotated 
    	AffineTransform orig = g2d.getTransform();
    	AffineTransform at = new AffineTransform();
        at.setToRotation( sprite.getRadianRotation() ,  center.x ,  center.y );     
        
        g2d.setTransform(at);
        g2d.drawImage(sprite.getImage(), location.x , location.y , null);
        g2d.setTransform(orig);
        
    }
    
    // initialitation with different sprite families
    // called from App
    public void initStupidSprites() {
 	   StupidSprite gekko = new StupidSprite(StupidSprite.GEKKO);
 	   StupidSprite turtle = new StupidSprite(StupidSprite.TURTLE);
 	   StupidSprite cat = new StupidSprite(StupidSprite.CAT);
       StupidSprite dog = new StupidSprite(StupidSprite.DOG , cat);

 	   gekko.setLocation(0, 100);
 	   turtle.setLocation(300,300);
 	   cat.setLocation(30,0);
 	   dog.setLocation(500,500);
 	   
 	   gekko.setRotation(-30);
 	   turtle.setRotation(350);
 	   cat.setRotation(90);
 	   dog.setRotation(180);
 	   
 	   gekko.setWorldInfo(world);
 	   turtle.setWorldInfo(world);
 	   cat.setWorldInfo(world);
 	   dog.setWorldInfo(world);
 	   
 	   sprites.add(gekko);
 	   sprites.add(turtle);
 	   sprites.add(cat);
 	   sprites.add(dog);	   
 	   
 	   
    }
       
    
    public void initBetterSprites() {
 	   Sprite gekko = new Gekko();
 	   Sprite turtle = new Turtle();
 	   Sprite cat = new Cat();
       Sprite dog = new Dog(cat);

 	   gekko.setLocation(0, 100);
 	   turtle.setLocation(300,300);
 	   cat.setLocation(30,0);
 	   dog.setLocation(500,500);
 	   
 	   gekko.setRotation(-30);
 	   turtle.setRotation(350);
 	   cat.setRotation(90);
 	   dog.setRotation(180);
 	   
 	   gekko.setWorldInfo(world);
 	   turtle.setWorldInfo(world);
 	   cat.setWorldInfo(world);
 	   dog.setWorldInfo(world);
 	   
 	   sprites.add(gekko);
 	   sprites.add(turtle);
 	   sprites.add(cat);
 	   sprites.add(dog);	   
 	   
    }  
    
    
    
    public void initBestSprites() {
  	   SmartSprite gekko = new GekkoSprite();
  	   SmartSprite turtle = new TurtleSprite();
       SmartSprite cat = new CatSprite();
  	   SmartSprite dog = new DogSprite(cat);

  	   gekko.setLocation(0, 100);
  	   turtle.setLocation(300,300);
  	   cat.setLocation(30,0);
  	   dog.setLocation(500,500);
  	   
  	   gekko.setRotation(-30);
  	   turtle.setRotation(350);
  	   cat.setRotation(90);
  	   dog.setRotation(180);
  	   
  	   gekko.setWorldInfo(world);
  	   turtle.setWorldInfo(world);
  	   cat.setWorldInfo(world);
  	   dog.setWorldInfo(world);
  	   

  	   
  	   sprites.add(gekko);
  	   sprites.add(turtle);
  	   
  	   // add without decoration
  	   // sprites.add(cat);
 	   // sprites.add(dog);
 	   
 	   // add with decoration:
  	   PenDecoration penDog = new PenDecoration (dog , Color.red);
  	   PenDecoration penCat = new PenDecoration (cat, Color.green);
  	   LogDecoration penAndLogDog = new LogDecoration (penDog);
  	   
  	   pens.add(penCat);
  	   pens.add(penDog);
 	   
       sprites.add(penCat);
       sprites.add(penAndLogDog);
  	   
     }   
    
    
    
    public void initSpritesWithFactory(AbstractSpriteFactory spriteFactory) {
    	
      
      SpriteInterface sprite1 = spriteFactory.createGekko(0, 100, -30, world);
      SpriteInterface sprite2 = spriteFactory.createTurtle(300,300,350,world);
      SpriteInterface sprite3 = spriteFactory.createCat(30,0,90,world);
      SpriteInterface sprite4 = spriteFactory.createDog(500,500,180,world,sprite3);
    	
 	   
 	   sprites.add(sprite1);
 	   sprites.add(sprite2);
 	   sprites.add(sprite3);
 	   sprites.add(sprite4);	   
 	   
 	   
    }
     
    
   
   public void initListeners() {
	   
       // frequently calls step for all sprites
	   Timer t = new Timer(100, new ActionListener () {
		@Override
		public void actionPerformed(ActionEvent arg0) {
	    	 Iterator <SpriteInterface> allSprites = sprites.iterator();
	    	 while (allSprites.hasNext()) {
	    		 allSprites.next().step();
	    	 }

			repaint(); // repaints whole component without optimization
		}
		   
	   });
	   
	   t.start();
	   
	   this.addMouseListener(new MouseListener() {

		@Override
		public void mouseClicked(MouseEvent arg0) {
			
		}

		@Override
		public void mouseEntered(MouseEvent arg0) {
			
		}

		@Override
		public void mouseExited(MouseEvent arg0) {
			
		}

		// mouse down sets new target in our world:
		@Override
		public void mousePressed(MouseEvent evt) {

			world.setTarget(new Point (evt.getX() , evt.getY()));
			
		}

		@Override
		public void mouseReleased(MouseEvent arg0) {
			
		}
		   
	   });
	   
	   
        // update word information on component resize
	    this.addComponentListener(new ComponentListener () {

			@Override
			public void componentHidden(ComponentEvent arg0) {
				
			}

			@Override
			public void componentMoved(ComponentEvent arg0) {
		
			}

			@Override
			public void componentResized(ComponentEvent evt) {
				Component c = evt.getComponent();
				world.setBounds(new Rectangle(0,0,600,600));
				world.setBounds(new Rectangle (0 , 0 , c.getWidth(),  c.getHeight()));
				
			}

			@Override
			public void componentShown(ComponentEvent arg0) {

			}
	    	
	    });
	   
   }
    

   
}
